@@ -43,6 +43,7 @@ class WorkspaceDialog(QObject):
         self._quality_name = ""
         self._num_settings_overridden_by_quality_changes = 0
         self._quality_type = ""
+        self._intent_name = ""
         self._machine_name = ""
         self._machine_type = ""
         self._variant_type = ""
@@ -60,6 +61,7 @@ class WorkspaceDialog(QObject):
     hasVisibleSettingsFieldChanged = pyqtSignal()
     numSettingsOverridenByQualityChangesChanged = pyqtSignal()
     qualityTypeChanged = pyqtSignal()
+    intentNameChanged = pyqtSignal()
     machineNameChanged = pyqtSignal()
     materialLabelsChanged = pyqtSignal()
     objectsOnPlateChanged = pyqtSignal()
@@ -166,6 +168,15 @@ class WorkspaceDialog(QObject):
             self._quality_name = quality_name
             self.qualityNameChanged.emit()
 
+    @pyqtProperty(str, notify = intentNameChanged)
+    def intentName(self) -> str:
+        return self._intent_name
+
+    def setIntentName(self, intent_name: str) -> None:
+        if self._intent_name != intent_name:
+            self._intent_name = intent_name
+            self.intentNameChanged.emit()
+
     @pyqtProperty(str, notify=activeModeChanged)
     def activeMode(self):
         return self._active_mode
@@ -218,9 +229,10 @@ class WorkspaceDialog(QObject):
         if key in self._result:
             self._result[key] = strategy
 
-    ##  Close the backend: otherwise one could end up with "Slicing..."
     @pyqtSlot()
     def closeBackend(self):
+        """Close the backend: otherwise one could end up with "Slicing..."""
+
         Application.getInstance().getBackend().close()
 
     def setMaterialConflict(self, material_conflict):
@@ -272,8 +284,9 @@ class WorkspaceDialog(QObject):
         self.showDialogSignal.emit()
 
     @pyqtSlot()
-    ##  Used to notify the dialog so the lock can be released.
     def notifyClosed(self):
+        """Used to notify the dialog so the lock can be released."""
+
         self._result = {} # The result should be cleared before hide, because after it is released the main thread lock
         self._visible = False
         try:
@@ -308,8 +321,9 @@ class WorkspaceDialog(QObject):
         self._view.hide()
         self.hide()
 
-    ##  Block thread until the dialog is closed.
     def waitForClose(self):
+        """Block thread until the dialog is closed."""
+
         if self._visible:
             if threading.current_thread() != threading.main_thread():
                 self._lock.acquire()
