@@ -51,7 +51,7 @@ class ThreeMFWorkspaceWriter(WorkspaceWriter):
                 self._writeContainerToArchive(container, archive)
 
             # Check if the machine has extruders and save all that data as well.
-            for extruder_stack in global_stack.extruders.values():
+            for extruder_stack in global_stack.extruderList:
                 self._writeContainerToArchive(extruder_stack, archive)
                 for container in extruder_stack.getContainers():
                     self._writeContainerToArchive(container, archive)
@@ -63,7 +63,7 @@ class ThreeMFWorkspaceWriter(WorkspaceWriter):
         # Write preferences to archive
         original_preferences = Application.getInstance().getPreferences() #Copy only the preferences that we use to the workspace.
         temp_preferences = Preferences()
-        for preference in {"general/visible_settings", "cura/active_mode", "cura/categories_expanded"}:
+        for preference in {"general/visible_settings", "cura/active_mode", "cura/categories_expanded", "metadata/setting_version"}:
             temp_preferences.addPreference(preference, None)
             temp_preferences.setValue(preference, original_preferences.getValue(preference))
         preferences_string = StringIO()
@@ -107,11 +107,13 @@ class ThreeMFWorkspaceWriter(WorkspaceWriter):
             import json
             archive.writestr(file_in_archive, json.dumps(metadata, separators = (", ", ": "), indent = 4, skipkeys = True))
 
-    ##  Helper function that writes ContainerStacks, InstanceContainers and DefinitionContainers to the archive.
-    #   \param container That follows the \type{ContainerInterface} to archive.
-    #   \param archive The archive to write to.
     @staticmethod
     def _writeContainerToArchive(container, archive):
+        """Helper function that writes ContainerStacks, InstanceContainers and DefinitionContainers to the archive.
+
+        :param container: That follows the :type{ContainerInterface} to archive.
+        :param archive: The archive to write to.
+        """
         if isinstance(container, type(ContainerRegistry.getInstance().getEmptyInstanceContainer())):
             return  # Empty file, do nothing.
 
@@ -125,15 +127,29 @@ class ThreeMFWorkspaceWriter(WorkspaceWriter):
 
         file_name = "Cura/%s.%s" % (container.getId(), file_suffix)
 
-        if file_name in archive.namelist():
-            return  # File was already saved, no need to do it again. Uranium guarantees unique ID's, so this should hold.
+        try:
+            if file_name in archive.namelist():
+                return  # File was already saved, no need to do it again. Uranium guarantees unique ID's, so this should hold.
 
-        file_in_archive = zipfile.ZipInfo(file_name)
-        # For some reason we have to set the compress type of each file as well (it doesn't keep the type of the entire archive)
-        file_in_archive.compress_type = zipfile.ZIP_DEFLATED
+            file_in_archive = zipfile.ZipInfo(file_name)
+            # For some reason we have to set the compress type of each file as well (it doesn't keep the type of the entire archive)
+            file_in_archive.compress_type = zipfile.ZIP_DEFLATED
 
-        # Do not include the network authentication keys
-        ignore_keys = {"network_authentication_id", "network_authentication_key", "octoprint_api_key"}
-        serialized_data = container.serialize(ignored_metadata_keys = ignore_keys)
+            # Do not include the network authentication keys
+            ignore_keys = {
+                "um_cloud_cluster_id",
+                "um_network_key",
+                "um_linked_to_account",
+                "removal_warning",
+                "host_guid",
+                "group_name",
+                "group_size",
+                "connection_type",
+                "octoprint_api_key"
+            }
+            serialized_data = container.serialize(ignored_metadata_keys = ignore_keys)
 
-        archive.writestr(file_in_archive, serialized_data)
+            archive.writestr(file_in_archive, serialized_data)
+        except (FileNotFoundError, EnvironmentError):
+            Logger.error("File became inaccessible while writing to it: {archive_filename}".format(archive_filename = archive.fp.name))
+            return
\ No newline at end of file
