@@ -8,11 +8,15 @@ import time
 from typing import Any, cast, Dict, List, Optional, Set
 import re
 import Arcus #For typing.
+from PyQt5.QtCore import QCoreApplication
 
 from UM.Job import Job
 from UM.Logger import Logger
 from UM.Scene.SceneNode import SceneNode
 from UM.Settings.ContainerStack import ContainerStack #For typing.
+from UM.Settings.InstanceContainer import InstanceContainer
+from UM.Settings.Interfaces import ContainerInterface
+from UM.Settings.SettingDefinition import SettingDefinition
 from UM.Settings.SettingRelation import SettingRelation #For typing.
 
 from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator
@@ -40,8 +44,9 @@ class StartJobResult(IntEnum):
     ObjectsWithDisabledExtruder = 8
 
 
-##  Formatter class that handles token expansion in start/end gcode
 class GcodeStartEndFormatter(Formatter):
+    """Formatter class that handles token expansion in start/end gcode"""
+
     def __init__(self, default_extruder_nr: int = -1) -> None:
         super().__init__()
         self._default_extruder_nr = default_extruder_nr
@@ -82,8 +87,9 @@ class GcodeStartEndFormatter(Formatter):
         return value
 
 
-##  Job class that builds up the message of scene data to send to CuraEngine.
 class StartSliceJob(Job):
+    """Job class that builds up the message of scene data to send to CuraEngine."""
+
     def __init__(self, slice_message: Arcus.PythonMessage) -> None:
         super().__init__()
 
@@ -100,27 +106,42 @@ class StartSliceJob(Job):
     def setBuildPlate(self, build_plate_number: int) -> None:
         self._build_plate_number = build_plate_number
 
-    ##  Check if a stack has any errors.
-    ##  returns true if it has errors, false otherwise.
     def _checkStackForErrors(self, stack: ContainerStack) -> bool:
-        if stack is None:
-            return False
+        """Check if a stack has any errors."""
 
-        # if there are no per-object settings we don't need to check the other settings here
-        stack_top = stack.getTop()
-        if stack_top is None or not stack_top.getAllKeys():
-            return False
+        """returns true if it has errors, false otherwise."""
 
-        for key in stack.getAllKeys():
-            validation_state = stack.getProperty(key, "validationState")
-            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError, ValidatorState.Invalid):
-                Logger.log("w", "Setting %s is not valid, but %s. Aborting slicing.", key, validation_state)
+        top_of_stack = cast(InstanceContainer, stack.getTop())  # Cache for efficiency.
+        changed_setting_keys = top_of_stack.getAllKeys()
+
+        # Add all relations to changed settings as well.
+        for key in top_of_stack.getAllKeys():
+            instance = top_of_stack.getInstance(key)
+            if instance is None:
+                continue
+            self._addRelations(changed_setting_keys, instance.definition.relations)
+            Job.yieldThread()
+
+        for changed_setting_key in changed_setting_keys:
+            validation_state = stack.getProperty(changed_setting_key, "validationState")
+
+            if validation_state is None:
+                definition = cast(SettingDefinition, stack.getSettingDefinition(changed_setting_key))
+                validator_type = SettingDefinition.getValidatorForType(definition.type)
+                if validator_type:
+                    validator = validator_type(changed_setting_key)
+                    validation_state = validator(stack)
+            if validation_state in (
+            ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError, ValidatorState.Invalid):
+                Logger.log("w", "Setting %s is not valid, but %s. Aborting slicing.", changed_setting_key, validation_state)
                 return True
             Job.yieldThread()
+
         return False
 
-    ##  Runs the job that initiates the slicing.
     def run(self) -> None:
+        """Runs the job that initiates the slicing."""
+
         if self._build_plate_number is None:
             self.setResult(StartJobResult.Error)
             return
@@ -236,7 +257,7 @@ class StartSliceJob(Job):
         global_stack = CuraApplication.getInstance().getGlobalContainerStack()
         if not global_stack:
             return
-        extruders_enabled = {position: stack.isEnabled for position, stack in global_stack.extruders.items()}
+        extruders_enabled = [stack.isEnabled for stack in global_stack.extruderList]
         filtered_object_groups = []
         has_model_with_disabled_extruders = False
         associated_disabled_extruders = set()
@@ -246,7 +267,7 @@ class StartSliceJob(Job):
             for node in group:
                 # Only check if the printing extruder is enabled for printing meshes
                 is_non_printing_mesh = node.callDecoration("evaluateIsNonPrintingMesh")
-                extruder_position = node.callDecoration("getActiveExtruderPosition")
+                extruder_position = int(node.callDecoration("getActiveExtruderPosition"))
                 if not is_non_printing_mesh and not extruders_enabled[extruder_position]:
                     skip_group = True
                     has_model_with_disabled_extruders = True
@@ -256,8 +277,8 @@ class StartSliceJob(Job):
 
         if has_model_with_disabled_extruders:
             self.setResult(StartJobResult.ObjectsWithDisabledExtruder)
-            associated_disabled_extruders = {str(c) for c in sorted([int(p) + 1 for p in associated_disabled_extruders])}
-            self.setMessage(", ".join(associated_disabled_extruders))
+            associated_disabled_extruders = {p + 1 for p in associated_disabled_extruders}
+            self.setMessage(", ".join(map(str, sorted(associated_disabled_extruders))))
             return
 
         # There are cases when there is nothing to slice. This can happen due to one at a time slicing not being
@@ -323,18 +344,17 @@ class StartSliceJob(Job):
     def setIsCancelled(self, value: bool):
         self._is_cancelled = value
 
-    ##  Creates a dictionary of tokens to replace in g-code pieces.
-    #
-    #   This indicates what should be replaced in the start and end g-codes.
-    #   \param stack The stack to get the settings from to replace the tokens
-    #   with.
-    #   \return A dictionary of replacement tokens to the values they should be
-    #   replaced with.
     def _buildReplacementTokens(self, stack: ContainerStack) -> Dict[str, Any]:
+        """Creates a dictionary of tokens to replace in g-code pieces.
+
+        This indicates what should be replaced in the start and end g-codes.
+        :param stack: The stack to get the settings from to replace the tokens with.
+        :return: A dictionary of replacement tokens to the values they should be replaced with.
+        """
+
         result = {}
         for key in stack.getAllKeys():
-            value = stack.getProperty(key, "value")
-            result[key] = value
+            result[key] = stack.getProperty(key, "value")
             Job.yieldThread()
 
         result["print_bed_temperature"] = result["material_bed_temperature"] # Renamed settings.
@@ -354,14 +374,18 @@ class StartSliceJob(Job):
         self._all_extruders_settings = {
             "-1": self._buildReplacementTokens(global_stack)
         }
+        QCoreApplication.processEvents()  # Ensure that the GUI does not freeze.
         for extruder_stack in ExtruderManager.getInstance().getActiveExtruderStacks():
             extruder_nr = extruder_stack.getProperty("extruder_nr", "value")
             self._all_extruders_settings[str(extruder_nr)] = self._buildReplacementTokens(extruder_stack)
+            QCoreApplication.processEvents()  # Ensure that the GUI does not freeze.
 
-    ##  Replace setting tokens in a piece of g-code.
-    #   \param value A piece of g-code to replace tokens in.
-    #   \param default_extruder_nr Stack nr to use when no stack nr is specified, defaults to the global stack
     def _expandGcodeTokens(self, value: str, default_extruder_nr: int = -1) -> str:
+        """Replace setting tokens in a piece of g-code.
+
+        :param value: A piece of g-code to replace tokens in.
+        :param default_extruder_nr: Stack nr to use when no stack nr is specified, defaults to the global stack
+        """
         if not self._all_extruders_settings:
             self._cacheAllExtruderSettings()
 
@@ -377,8 +401,9 @@ class StartSliceJob(Job):
             Logger.logException("w", "Unable to do token replacement on start/end g-code")
             return str(value)
 
-    ##  Create extruder message from stack
     def _buildExtruderMessage(self, stack: ContainerStack) -> None:
+        """Create extruder message from stack"""
+
         message = self._slice_message.addRepeatedMessage("extruders")
         message.id = int(stack.getMetaDataEntry("position"))
         if not self._all_extruders_settings:
@@ -398,20 +423,27 @@ class StartSliceJob(Job):
         settings["machine_extruder_start_code"] = self._expandGcodeTokens(settings["machine_extruder_start_code"], extruder_nr)
         settings["machine_extruder_end_code"] = self._expandGcodeTokens(settings["machine_extruder_end_code"], extruder_nr)
 
+        global_definition = cast(ContainerInterface, cast(ContainerStack, stack.getNextStack()).getBottom())
+        own_definition = cast(ContainerInterface, stack.getBottom())
+
         for key, value in settings.items():
             # Do not send settings that are not settable_per_extruder.
-            if not stack.getProperty(key, "settable_per_extruder"):
-                continue
+            # Since these can only be set in definition files, we only have to ask there.
+            if not global_definition.getProperty(key, "settable_per_extruder") and \
+                    not own_definition.getProperty(key, "settable_per_extruder"):
+                    continue
             setting = message.getMessage("settings").addRepeatedMessage("settings")
             setting.name = key
             setting.value = str(value).encode("utf-8")
             Job.yieldThread()
 
-    ##  Sends all global settings to the engine.
-    #
-    #   The settings are taken from the global stack. This does not include any
-    #   per-extruder settings or per-object settings.
     def _buildGlobalSettingsMessage(self, stack: ContainerStack) -> None:
+        """Sends all global settings to the engine.
+
+        The settings are taken from the global stack. This does not include any
+        per-extruder settings or per-object settings.
+        """
+
         if not self._all_extruders_settings:
             self._cacheAllExtruderSettings()
 
@@ -422,6 +454,8 @@ class StartSliceJob(Job):
 
         # Pre-compute material material_bed_temp_prepend and material_print_temp_prepend
         start_gcode = settings["machine_start_gcode"]
+        # Remove all the comments from the start g-code
+        start_gcode = re.sub(r";.+?(\n|$)", "\n", start_gcode)
         bed_temperature_settings = ["material_bed_temperature", "material_bed_temperature_layer_0"]
         pattern = r"\{(%s)(,\s?\w+)?\}" % "|".join(bed_temperature_settings) # match {setting} as well as {setting, extruder_nr}
         settings["material_bed_temp_prepend"] = re.search(pattern, start_gcode) == None
@@ -431,9 +465,7 @@ class StartSliceJob(Job):
 
         # Replace the setting tokens in start and end g-code.
         # Use values from the first used extruder by default so we get the expected temperatures
-        initial_extruder_stack = CuraApplication.getInstance().getExtruderManager().getUsedExtruderStacks()[0]
-        initial_extruder_nr = initial_extruder_stack.getProperty("extruder_nr", "value")
-
+        initial_extruder_nr = CuraApplication.getInstance().getExtruderManager().getInitialExtruderNr()
         settings["machine_start_gcode"] = self._expandGcodeTokens(settings["machine_start_gcode"], initial_extruder_nr)
         settings["machine_end_gcode"] = self._expandGcodeTokens(settings["machine_end_gcode"], initial_extruder_nr)
 
@@ -444,15 +476,16 @@ class StartSliceJob(Job):
             setting_message.value = str(value).encode("utf-8")
             Job.yieldThread()
 
-    ##  Sends for some settings which extruder they should fallback to if not
-    #   set.
-    #
-    #   This is only set for settings that have the limit_to_extruder
-    #   property.
-    #
-    #   \param stack The global stack with all settings, from which to read the
-    #   limit_to_extruder property.
     def _buildGlobalInheritsStackMessage(self, stack: ContainerStack) -> None:
+        """Sends for some settings which extruder they should fallback to if not set.
+
+        This is only set for settings that have the limit_to_extruder
+        property.
+
+        :param stack: The global stack with all settings, from which to read the
+            limit_to_extruder property.
+        """
+
         for key in stack.getAllKeys():
             extruder_position = int(round(float(stack.getProperty(key, "limit_to_extruder"))))
             if extruder_position >= 0:  # Set to a specific extruder.
@@ -461,10 +494,13 @@ class StartSliceJob(Job):
                 setting_extruder.extruder = extruder_position
             Job.yieldThread()
 
-    ##  Check if a node has per object settings and ensure that they are set correctly in the message
-    #   \param node Node to check.
-    #   \param message object_lists message to put the per object settings in
     def _handlePerObjectSettings(self, node: CuraSceneNode, message: Arcus.PythonMessage):
+        """Check if a node has per object settings and ensure that they are set correctly in the message
+
+        :param node: Node to check.
+        :param message: object_lists message to put the per object settings in
+        """
+
         stack = node.callDecoration("getStack")
 
         # Check if the node has a stack attached to it and the stack has any settings in the top container.
@@ -500,14 +536,16 @@ class StartSliceJob(Job):
 
             Job.yieldThread()
 
-    ##  Recursive function to put all settings that require each other for value changes in a list
-    #   \param relations_set Set of keys of settings that are influenced
-    #   \param relations list of relation objects that need to be checked.
     def _addRelations(self, relations_set: Set[str], relations: List[SettingRelation]):
+        """Recursive function to put all settings that require each other for value changes in a list
+
+        :param relations_set: Set of keys of settings that are influenced
+        :param relations: list of relation objects that need to be checked.
+        """
+
         for relation in filter(lambda r: r.role == "value" or r.role == "limit_to_extruder", relations):
             if relation.type == RelationType.RequiresTarget:
                 continue
 
             relations_set.add(relation.target.key)
             self._addRelations(relations_set, relation.target.relations)
-
