@@ -16,23 +16,27 @@ from cura.OAuth2.Models import AuthenticationResponse, UserProfile, OAuth2Settin
 catalog = i18nCatalog("cura")
 TOKEN_TIMESTAMP_FORMAT = "%Y-%m-%d %H:%M:%S"
 
-##  Class containing several helpers to deal with the authorization flow.
 class AuthorizationHelpers:
+    """Class containing several helpers to deal with the authorization flow."""
+
     def __init__(self, settings: "OAuth2Settings") -> None:
         self._settings = settings
         self._token_url = "{}/token".format(self._settings.OAUTH_SERVER_URL)
 
     @property
-    ##  The OAuth2 settings object.
     def settings(self) -> "OAuth2Settings":
+        """The OAuth2 settings object."""
+
         return self._settings
 
-    ##  Request the access token from the authorization server.
-    #   \param authorization_code: The authorization code from the 1st step.
-    #   \param verification_code: The verification code needed for the PKCE
-    #   extension.
-    #   \return An AuthenticationResponse object.
     def getAccessTokenUsingAuthorizationCode(self, authorization_code: str, verification_code: str) -> "AuthenticationResponse":
+        """Request the access token from the authorization server.
+
+        :param authorization_code: The authorization code from the 1st step.
+        :param verification_code: The verification code needed for the PKCE extension.
+        :return: An AuthenticationResponse object.
+        """
+
         data = {
             "client_id": self._settings.CLIENT_ID if self._settings.CLIENT_ID is not None else "",
             "redirect_uri": self._settings.CALLBACK_URL if self._settings.CALLBACK_URL is not None else "",
@@ -46,10 +50,13 @@ class AuthorizationHelpers:
         except requests.exceptions.ConnectionError:
             return AuthenticationResponse(success=False, err_message="Unable to connect to remote server")
 
-    ##  Request the access token from the authorization server using a refresh token.
-    #   \param refresh_token:
-    #   \return An AuthenticationResponse object.
     def getAccessTokenUsingRefreshToken(self, refresh_token: str) -> "AuthenticationResponse":
+        """Request the access token from the authorization server using a refresh token.
+
+        :param refresh_token:
+        :return: An AuthenticationResponse object.
+        """
+
         Logger.log("d", "Refreshing the access token.")
         data = {
             "client_id": self._settings.CLIENT_ID if self._settings.CLIENT_ID is not None else "",
@@ -64,10 +71,13 @@ class AuthorizationHelpers:
             return AuthenticationResponse(success=False, err_message="Unable to connect to remote server")
 
     @staticmethod
-    ##  Parse the token response from the authorization server into an AuthenticationResponse object.
-    #   \param token_response: The JSON string data response from the authorization server.
-    #   \return An AuthenticationResponse object.
     def parseTokenResponse(token_response: requests.models.Response) -> "AuthenticationResponse":
+        """Parse the token response from the authorization server into an AuthenticationResponse object.
+
+        :param token_response: The JSON string data response from the authorization server.
+        :return: An AuthenticationResponse object.
+        """
+
         token_data = None
 
         try:
@@ -89,10 +99,13 @@ class AuthorizationHelpers:
                                       scope=token_data["scope"],
                                       received_at=datetime.now().strftime(TOKEN_TIMESTAMP_FORMAT))
 
-    ##  Calls the authentication API endpoint to get the token data.
-    #   \param access_token: The encoded JWT token.
-    #   \return Dict containing some profile data.
     def parseJWT(self, access_token: str) -> Optional["UserProfile"]:
+        """Calls the authentication API endpoint to get the token data.
+
+        :param access_token: The encoded JWT token.
+        :return: Dict containing some profile data.
+        """
+
         try:
             token_request = requests.get("{}/check-token".format(self._settings.OAUTH_SERVER_URL), headers = {
                 "Authorization": "Bearer {}".format(access_token)
@@ -115,16 +128,22 @@ class AuthorizationHelpers:
         )
 
     @staticmethod
-    ##  Generate a verification code of arbitrary length.
-    #   \param code_length: How long should the code be? This should never be lower than 16, but it's probably better to
-    #   leave it at 32
     def generateVerificationCode(code_length: int = 32) -> str:
+        """Generate a verification code of arbitrary length.
+
+        :param code_length:: How long should the code be? This should never be lower than 16, but it's probably
+        better to leave it at 32
+        """
+
         return "".join(random.choice("0123456789ABCDEF") for i in range(code_length))
 
     @staticmethod
-    ##  Generates a base64 encoded sha512 encrypted version of a given string.
-    #   \param verification_code:
-    #   \return The encrypted code in base64 format.
     def generateVerificationCodeChallenge(verification_code: str) -> str:
+        """Generates a base64 encoded sha512 encrypted version of a given string.
+
+        :param verification_code:
+        :return: The encrypted code in base64 format.
+        """
+
         encoded = sha512(verification_code.encode()).digest()
         return b64encode(encoded, altchars = b"_-").decode()
