@@ -22,10 +22,9 @@ if TYPE_CHECKING:
     from cura.Settings.GlobalStack import GlobalStack
 
 
-##  Represents an Extruder and its related containers.
-#
-#
 class ExtruderStack(CuraContainerStack):
+    """Represents an Extruder and its related containers."""
+
     def __init__(self, container_id: str) -> None:
         super().__init__(container_id)
 
@@ -33,24 +32,29 @@ class ExtruderStack(CuraContainerStack):
 
         self.propertiesChanged.connect(self._onPropertiesChanged)
 
+        self.setDirty(False)
+
     enabledChanged = pyqtSignal()
 
-    ##  Overridden from ContainerStack
-    #
-    #   This will set the next stack and ensure that we register this stack as an extruder.
     @override(ContainerStack)
     def setNextStack(self, stack: CuraContainerStack, connect_signals: bool = True) -> None:
+        """Overridden from ContainerStack
+
+        This will set the next stack and ensure that we register this stack as an extruder.
+        """
+
         super().setNextStack(stack)
         stack.addExtruder(self)
         self.setMetaDataEntry("machine", stack.id)
 
-        # For backward compatibility: Register the extruder with the Extruder Manager
-        ExtruderManager.getInstance().registerExtruder(self, stack.id)
-
     @override(ContainerStack)
     def getNextStack(self) -> Optional["GlobalStack"]:
         return super().getNextStack()
 
+    @pyqtProperty(int, constant = True)
+    def position(self) -> int:
+        return int(self.getMetaDataEntry("position"))
+
     def setEnabled(self, enabled: bool) -> None:
         if self.getMetaDataEntry("enabled", True) == enabled: # No change.
             return # Don't emit a signal then.
@@ -67,11 +71,13 @@ class ExtruderStack(CuraContainerStack):
 
     compatibleMaterialDiameterChanged = pyqtSignal()
 
-    ##  Return the filament diameter that the machine requires.
-    #
-    #   If the machine has no requirement for the diameter, -1 is returned.
-    #   \return The filament diameter for the printer
     def getCompatibleMaterialDiameter(self) -> float:
+        """Return the filament diameter that the machine requires.
+
+        If the machine has no requirement for the diameter, -1 is returned.
+        :return: The filament diameter for the printer
+        """
+
         context = PropertyEvaluationContext(self)
         context.context["evaluate_from_container_index"] = _ContainerIndexes.Variant
 
@@ -93,31 +99,35 @@ class ExtruderStack(CuraContainerStack):
 
     approximateMaterialDiameterChanged = pyqtSignal()
 
-    ##  Return the approximate filament diameter that the machine requires.
-    #
-    #   The approximate material diameter is the material diameter rounded to
-    #   the nearest millimetre.
-    #
-    #   If the machine has no requirement for the diameter, -1 is returned.
-    #
-    #   \return The approximate filament diameter for the printer
     def getApproximateMaterialDiameter(self) -> float:
+        """Return the approximate filament diameter that the machine requires.
+
+        The approximate material diameter is the material diameter rounded to
+        the nearest millimetre.
+
+        If the machine has no requirement for the diameter, -1 is returned.
+
+        :return: The approximate filament diameter for the printer
+        """
+
         return round(self.getCompatibleMaterialDiameter())
 
     approximateMaterialDiameter = pyqtProperty(float, fget = getApproximateMaterialDiameter,
                                                notify = approximateMaterialDiameterChanged)
 
-    ##  Overridden from ContainerStack
-    #
-    #   It will perform a few extra checks when trying to get properties.
-    #
-    #   The two extra checks it currently does is to ensure a next stack is set and to bypass
-    #   the extruder when the property is not settable per extruder.
-    #
-    #   \throws Exceptions.NoGlobalStackError Raised when trying to get a property from an extruder without
-    #                                         having a next stack set.
     @override(ContainerStack)
     def getProperty(self, key: str, property_name: str, context: Optional[PropertyEvaluationContext] = None) -> Any:
+        """Overridden from ContainerStack
+
+        It will perform a few extra checks when trying to get properties.
+
+        The two extra checks it currently does is to ensure a next stack is set and to bypass
+        the extruder when the property is not settable per extruder.
+
+        :throws Exceptions.NoGlobalStackError Raised when trying to get a property from an extruder without
+        having a next stack set.
+        """
+
         if not self._next_stack:
             raise Exceptions.NoGlobalStackError("Extruder {id} is missing the next stack!".format(id = self.id))
 
@@ -135,12 +145,15 @@ class ExtruderStack(CuraContainerStack):
             if limit_to_extruder == -1:
                 limit_to_extruder = int(cura.CuraApplication.CuraApplication.getInstance().getMachineManager().defaultExtruderPosition)
             limit_to_extruder = str(limit_to_extruder)
+
         if (limit_to_extruder is not None and limit_to_extruder != "-1") and self.getMetaDataEntry("position") != str(limit_to_extruder):
-            if str(limit_to_extruder) in self.getNextStack().extruders:
-                result = self.getNextStack().extruders[str(limit_to_extruder)].getProperty(key, property_name, context)
+            try:
+                result = self.getNextStack().extruderList[int(limit_to_extruder)].getProperty(key, property_name, context)
                 if result is not None:
                     context.popContainer()
                     return result
+            except IndexError:
+                pass
 
         result = super().getProperty(key, property_name, context)
         context.popContainer()
